float4x4 matWorldViewProj;	
float4x4 matWorld;	
float4 vecLightDir;
float4 vecEye;
bool	bSpecular;
float	fTile;
float	fFadeAmnt;
float	fAlpha;

// diffuse color map texture
texture ColorMap;
sampler ColorMapSampler = sampler_state
{
   Texture = <ColorMap>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
};

// bumpmap texture
texture BumpMap;
sampler BumpMapSampler = sampler_state
{
   Texture = <BumpMap>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
};

// definer output
struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float2 Tex : TEXCOORD0;
    float3 Light : TEXCOORD1;
    float3 View : TEXCOORD2;
};

// Vertex shader
VS_OUTPUT VS(float4 Pos : POSITION, float2 Tex : TEXCOORD, float3 Normal : NORMAL, float3 Tangent : TANGENT  )
{
    VS_OUTPUT Out = (VS_OUTPUT)0;      
    Out.Pos = mul(Pos, matWorldViewProj);	// posisjonen til vertex
    
    // worldspace -> tangent space
    float3x3 worldToTangentSpace;
    worldToTangentSpace[0] = mul(Tangent, matWorld);
    worldToTangentSpace[1] = mul(cross(Tangent, Normal), matWorld);
    worldToTangentSpace[2] = mul(Normal, matWorld);


    Tex.x *= fTile;
    Tex.y *= fTile;
    
    Out.Tex = Tex;
    
    
    float4 PosWorld = mul(Pos, matWorld);	
    
    Out.Light = mul(worldToTangentSpace, vecLightDir);
    Out.View = mul(worldToTangentSpace, vecEye - PosWorld);
    
   return Out;
}

// Pixel Shader
float4 PS(float2 Tex: TEXCOORD0, float3 Light : TEXCOORD1, float3 View : TEXCOORD2) : COLOR
{
    float4 Ambient = float4(0.2, 0.2, 0.2, 1.0);
    
    // Hent fargen p gjeldende pixel  ( C )
    float4 Color = tex2D(ColorMapSampler, Tex);	
    float4 BumpMap = tex2D(BumpMapSampler, Tex);	
    float4 retColor = lerp(Color, BumpMap, fFadeAmnt);

    float4 ret;
	ret = retColor;

	ret.a *= fAlpha;
	
	return ret;
}

// -------------------------------------------------------------
// 
// -------------------------------------------------------------
technique TShader
{
    pass P0
    {
        Sampler[0] = (ColorMapSampler);		
        Sampler[1] = (BumpMapSampler);		
    
        // compile shaders
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}
